import os
import sys
import json
from subprocess import Popen, PIPE
import subprocess
import time
import traceback

NUM_TESTS = { 'SinglyLinkedList' : 7,
              'BinaryTree': 6,
              'GraphMatrix': 8}

def get_output(code_string, java_user_class):

	java_user_file =  java_user_class + '.java'
	output_json = []
	with open(java_user_file, 'w+') as f:
		f.write(code_string)
	os.system('rm ' + java_user_class + '.class')

	try:
		subprocess.check_output(['javac ' + java_user_file + ' 2> stderr.txt'], shell=True)
	except subprocess.CalledProcessError as e:
		with open('stderr.txt', 'r') as file:
			compile_error = file.read()
		for i in range(1, NUM_TESTS[java_user_class] + 1):
			output_json.append({"test_number" : str(i),
				"pass" : False,
				"output" : compile_error})

		return json.dumps(output_json)
	
	running_procs = []
	for i in range(1, NUM_TESTS[java_user_class] + 1):
		print('iter', i)
		this_test_file = java_user_class + 'Test' + str(i)
		this_correct_file = this_test_file + 'correct.txt'
		ps = Popen(['java','-cp', '.', this_test_file], stdout=PIPE, stderr=PIPE)
		running_procs.append({"correct_file" : this_correct_file,
			"popen": ps,
			"test_number": str(i)})


	procs_start = time.time()
	while running_procs:
		if (time.time() - procs_start) > 30:
			for i in range(1, NUM_TESTS[java_user_class] + 1):
				output_json.append({"test_number" : str(i),
					"pass" : False,
					"output" : "Infinite Loop Occurred"})
		for proc in running_procs:
			pop = proc['popen']
			retcode = pop.poll()
			if retcode is not None: # Process finished.
				if retcode != 0:
					error_output = pop.communicate()[1].decode('utf-8')
					print('error output', error_output)
					output_json.append({'test_number': proc['test_number'],
						                'pass' : False,
						                'output': error_output})
					running_procs.remove(proc)
				else:
					update_output_json(proc, output_json)
					running_procs.remove(proc)
			else: # No process is done, wait a bit and check again.
				time.sleep(.1)
				continue



	return json.dumps(output_json)

def main():

	print(get_output(TEST_CODE_STRING))


if __name__ == "__main__":
	main()


def update_output_json(proc, output_json):
	test_output = proc['popen'].communicate()[0].decode('utf-8')
	with open(proc["correct_file"], 'r') as f_correct:
		correct_output = f_correct.read()
		is_output_correct = (test_output == correct_output)

		output_json.append({
			'test_number': proc['test_number'],
			'pass': is_output_correct,
			'actual_output': test_output,
			'correct_output': correct_output			
		})
